@@ -67,8 +67,12 @@ struct dcesrv_interface {
 
 	/* for any private use by the interface code */
 	const void *private_data;
+
+	uint64_t flags;
 };
 
+#define DCESRV_INTERFACE_FLAGS_HANDLES_NOT_USED 0x00000001
+
 enum dcesrv_call_list {
 	DCESRV_LIST_NONE,
 	DCESRV_LIST_CALL_LIST,
@@ -76,6 +80,11 @@ enum dcesrv_call_list {
 	DCESRV_LIST_PENDING_CALL_LIST
 };
 
+struct data_blob_list_item {
+	struct data_blob_list_item *prev,*next;
+	DATA_BLOB blob;
+};
+
 /* the state of an ongoing dcerpc call */
 struct dcesrv_call_state {
 	struct dcesrv_call_state *next, *prev;
@@ -83,6 +92,11 @@ struct dcesrv_call_state {
 	struct dcesrv_connection_context *context;
 	struct ncacn_packet pkt;
 
+	/*
+	 * Used during async bind/alter_context.
+	 */
+	struct ncacn_packet ack_pkt;
+
 	/*
 	  which list this request is in, if any
 	 */
@@ -140,6 +154,32 @@ struct dcesrv_call_state {
 	struct dcerpc_auth *out_auth_info;
 };
 
+/*
+* DCERPC Handles
+* --------------
+* The various handles that are used in the RPC servers should be
+* created and fetch using the dcesrv_handle_* functions.
+*
+* Use
+* dcesrv_handle_new(struct dcesrv_connection \*, uint8 handle_type)
+* to obtain a new handle of the specified type. Handle types are
+* unique within each pipe.
+*
+* The handle can later be fetched again using:
+*
+* struct dcesrv_handle *dcesrv_handle_fetch(
+*         struct dcesrv_connection *dce_conn,
+*         struct policy_handle *p,
+*         uint8 handle_type)
+*
+* and destroyed by:
+*
+* 	dcesrv_handle_destroy(struct dcesrv_handle *).
+*
+* User data should be stored in the 'data' member of the dcesrv_handle
+* struct.
+*/
+
 #define DCESRV_HANDLE_ANY 255
 
 /* a dcerpc handle in internal format */
@@ -168,10 +208,7 @@ struct dcesrv_auth {
 
 struct dcesrv_connection_context {
 	struct dcesrv_connection_context *next, *prev;
-	uint32_t context_id;
-
-	/* TODO: remove this legacy (for openchange) in master */
-	struct dcesrv_assoc_group *assoc_group;
+	uint16_t context_id;
 
 	/* the connection this is on */
 	struct dcesrv_connection *conn;
@@ -187,6 +224,9 @@ struct dcesrv_connection_context {
 	 */
 	enum dcerpc_AuthLevel min_auth_level;
 	bool allow_connect;
+
+	/* the negotiated transfer syntax */
+	struct ndr_syntax_id transfer_syntax;
 };
 
 
@@ -219,16 +259,6 @@ struct dcesrv_connection {
 
 	DATA_BLOB partial_input;
 
-	/* This can be removed in master... */
-	struct  {
-		struct dcerpc_auth *auth_info;
-		struct gensec_security *gensec_security;
-		struct auth_session_info *session_info;
-		NTSTATUS (*session_key)(struct dcesrv_connection *, DATA_BLOB *session_key);
-		bool client_hdr_signing;
-		bool hdr_signing;
-	} _unused_auth_state;
-
 	/* the event_context that will be used for this connection */
 	struct tevent_context *event_ctx;
 
@@ -276,6 +306,24 @@ struct dcesrv_connection {
 
 	/* The maximum total payload of reassembled request pdus */
 	size_t max_total_request_size;
+
+	/*
+	 * Our preferred transfer syntax.
+	 */
+	const struct ndr_syntax_id *preferred_transfer;
+
+	/* the negotiated bind time features */
+	uint16_t bind_time_features;
+
+	/*
+	 * This is used to block the connection during
+	 * pending authentication.
+	 */
+	struct tevent_req *(*wait_send)(TALLOC_CTX *mem_ctx,
+					struct tevent_context *ev,
+					void *private_data);
+	NTSTATUS (*wait_recv)(struct tevent_req *req);
+	void *wait_private;
 };
 
 
@@ -340,6 +388,12 @@ struct dcesrv_context {
 			struct dcesrv_if_list *next, *prev;
 			struct dcesrv_interface iface;
 		} *interface_list;
+
+		/*
+		 * Should this service be run in a single process (so far only
+		 * NETLOGON is not run in a single process)
+		 */
+		bool use_single_process;
 	} *endpoint_list;
 
 	/* loadparm context to use for this connection */
@@ -370,7 +424,7 @@ NTSTATUS dcesrv_interface_register(struct dcesrv_context *dce_ctx,
 				   const char *ep_name,
 				   const struct dcesrv_interface *iface,
 				   const struct security_descriptor *sd);
-NTSTATUS dcerpc_register_ep_server(const void *_ep_server);
+NTSTATUS dcerpc_register_ep_server(const struct dcesrv_endpoint_server *ep_server);
 NTSTATUS dcesrv_init_context(TALLOC_CTX *mem_ctx, 
 				      struct loadparm_context *lp_ctx,
 				      const char **endpoint_servers, struct dcesrv_context **_dce_ctx);
@@ -392,9 +446,7 @@ struct dcesrv_handle *dcesrv_handle_fetch(
 					  struct dcesrv_connection_context *context, 
 					  struct policy_handle *p,
 					  uint8_t handle_type);
-struct socket_address *dcesrv_connection_get_my_addr(struct dcesrv_connection *conn, TALLOC_CTX *mem_ctx);
 
-struct socket_address *dcesrv_connection_get_peer_addr(struct dcesrv_connection *conn, TALLOC_CTX *mem_ctx);
 const struct tsocket_address *dcesrv_connection_get_local_address(struct dcesrv_connection *conn);
 const struct tsocket_address *dcesrv_connection_get_remote_address(struct dcesrv_connection *conn);
 
@@ -435,13 +487,14 @@ NTSTATUS dcesrv_fetch_session_key(struct dcesrv_connection *p, DATA_BLOB *sessio
 } while (0)
 
 #define DCESRV_PULL_HANDLE(h, inhandle, t) DCESRV_PULL_HANDLE_RETVAL(h, inhandle, t, NT_STATUS_INVALID_HANDLE)
-#define DCESRV_PULL_HANDLE_WERR(h, inhandle, t) DCESRV_PULL_HANDLE_RETVAL(h, inhandle, t, WERR_BADFID)
+#define DCESRV_PULL_HANDLE_WERR(h, inhandle, t) DCESRV_PULL_HANDLE_RETVAL(h, inhandle, t, WERR_INVALID_HANDLE)
 
 NTSTATUS dcesrv_add_ep(struct dcesrv_context *dce_ctx,
 		       struct loadparm_context *lp_ctx,
 		       struct dcesrv_endpoint *e,
 		       struct tevent_context *event_ctx,
-		       const struct model_ops *model_ops);
+		       const struct model_ops *model_ops,
+		       void *process_context);
 
 /**
  * retrieve credentials from a dce_call
